/*
   
  v. 0.543
*/

#ifndef __PARSERTOOLS_H__
#define __PARSERTOOLS_H__

#include <string>
#include <fstream>
#include <vector>
#include <iostream>
#include <ctime>

using namespace std;

class SrcReader {
        private:
                ifstream iFile;                         
                vector <string> iEndLineComments;
                int iLine;
                bool lastReadedLine; //  ,       ,   
                string iFileName;               
                bool __ReadWord(string &aStr);
                
        public:
                SrcReader();
                ~SrcReader();
                bool Open(const string &aFileName);
                void AddEndLineComment(const string &aComment);
                bool ReadLine(string &aStr);
                bool ReadWord(string &aStr);
                string ReadWord();      //     ,  
                void Error(const string &aMessage);
                int GetInt(const string &aStr);
                bool VerifyWord(const string &aStr, bool aIgnoreCase = false);
                //  ,      ,     false
                //  aIgnoreCase ,   
                


                bool errorFlag;
                string strSpacesChars;  //   
                string strDelimsChars;  // ,   
                string strWordChars;    // ,                       
};


//     
class StringReader {
        string str;
        int pos;

        StringReader() {};
public:
        string strSpacesChars;  //   
        string strDelimsChars;  // ,   


        StringReader(string aStr): str(aStr), pos(0) {
                strSpacesChars = " \t\x0a\x0d";
                strDelimsChars = ";,{}()";      
        };

        // true -   ,  -  
        bool ReadWord(string &aStr);
};



class BinReader {
        private:
                ifstream iFile;
        public:
                BinReader();
                ~BinReader();
                bool Open(const string &aFileName);
                void Read(const void *aP, const int aSize);
                unsigned int ReadByte();
                unsigned int ReadWord();
                unsigned int ReadDword();
                string ReadString(const int aLen);              
                bool eof();
                unsigned int size();
};



class BinWriter {
        //private:
                ofstream iFile;

        public:
                BinWriter();
                ~BinWriter();
                bool Open(const string &aFileName);
                void Write(const void *aP, const int aSize);
                void WriteByte(const unsigned int aVal);
                void Write3Byte(const unsigned int aVal);
                void WriteWord(const unsigned int aVal);
                void WriteDword(const unsigned int aVal);
                void WriteString(const string &aStr, const int aLen);

                void Seek(long aPos, ios_base::seekdir aDir) {                  
                        iFile.seekp(aPos, aDir);
                }

};




//          
string Trim(const string &aStr);

//        
int WordCount(const string &aStr, const string &aWordDelims);

//        -1,    
int WordPosition(const int aN, const string &aStr, const string &aWordDelims);

//     
string ExtractWord(const int aN, const string &aStr, const string &aWordDelims);

//   ,    
//  true,      
bool DeleteApostrof(string &aStr);

//    
inline int StrToInt(const string &aStr) {
        return atoi(aStr.c_str());
}

// ,     
bool VerifyInt(const string &aStr);

//     
inline double StrToFloat(const string &aStr) {
        return atof(aStr.c_str());
}

// ,      
bool VerifyFloat(const string &aStr);

//  hex   (2 !)   
//  -1/-2    
int HexToByte(const string &aStr);

//    hex-
string ByteToHex(int aVal);


bool VerifyDate(const string &aStr);
// ,        "DD.MM.YYYY"
bool VerifyTime(const string &aStr);
// ,        "HH:MM:SS"
bool VerifyDateTime(const string &aStr);
// ,          "DD.MM.YYYY HH:MM:SS"

tm StrToDate(const string &aStr);
//   
tm StrToTime(const string &aStr);
//   
tm StrToDateTime(const string &aStr);
//   /



//   /  - ,   1.01.1970 0:00:00 GMT

//           (ANSI)

inline string StrUpper(const string &aStr) {
  std::string result;

  result.resize(aStr.length());
  for (size_t i = 0; i < aStr.length(); i++)
    result[i] = toupper(aStr[i]);
  return result;

};


//           (ANSI)
inline string StrLower(const string &aStr) {
  std::string result;

  result.resize(aStr.length());
  for (size_t i = 0; i < aStr.length(); i++)
    result[i] = tolower(aStr[i]);
  return result;
};



#endif

